@@ -18,9 +18,8 @@
 #    define DEFAULT_BUFFER_SIZE (1024*1024)
 #endif
 
-GtkTreeView	*accel_tview = NULL;
 GtkListStore	*accel_store = NULL;
-GtkAccelGroup	*accel_group = NULL;
+GtkListStore    *special_accel_store = NULL;
 
 int		debug = 0;
 double          zoom_factor = 1.0;
@@ -36,97 +35,75 @@ typedef struct {
 
 #ifdef USE_HILDON
 key_assignment key_asg0[] = {
-  {N_("Zoom in"),	"zoom_in_btn",
-			0,		HILDON_HARDKEY_INCREASE},
-  {N_("Zoom out"),	"zoom_out_btn",
-			0,		HILDON_HARDKEY_DECREASE},
-  {N_("Fit width"),	"zoom_width_btn",
-			0,		HILDON_HARDKEY_RIGHT},
-  {N_("Fit page"),	"zoom_fit_btn",
-			0,		HILDON_HARDKEY_LEFT},
-  {N_("Zoom 1:1"),	"zoom_orig_btn",
-			0,		GDK_z},
-  {N_("Next page"),	"next_btn",
-			0,		HILDON_HARDKEY_DOWN},
-  {N_("Previous page"),	"back_btn",
-			0,		HILDON_HARDKEY_UP},
-  {N_("Last page"),	"last_btn",
-			0,		GDK_l},
-  {N_("First page"),	"begin_btn",
-			0,		GDK_f},
-  {N_("Open file"),	"open_btn",
-			0,	        0},
-  {N_("Quit cbrPager"),	"quit_btn",
-                        0,		0},
-  {N_("Rotate"),	"rotate_btn",
-                        0,		HILDON_HARDKEY_ESC},
-  {N_("Full screen"),    "fullscreen_btn",
-                        0,              HILDON_HARDKEY_FULLSCREEN},
-  {N_("Toggle mode"),   "toggle_btn",
-                        0,              HILDON_HARDKEY_SELECT},
+  {N_("Zoom in"),	"zoom_in_btn",    0, HILDON_HARDKEY_INCREASE},
+  {N_("Zoom out"),	"zoom_out_btn",   0, HILDON_HARDKEY_DECREASE},
+  {N_("Fit width"),	"zoom_width_btn", 0, 0},
+  {N_("Fit page"),	"zoom_fit_btn",   0, 0},
+  {N_("Zoom 1:1"),	"zoom_orig_btn",  0, GDK_z},
+  {N_("Next page"),	"next_btn",       0, HILDON_HARDKEY_DOWN},
+  {N_("Previous page"),	"back_btn",       0, HILDON_HARDKEY_UP},
+  {N_("Last page"),	"last_btn",       0, GDK_l},
+  {N_("First page"),	"begin_btn",      0, GDK_f},
+  {N_("Open file"),	"open_btn",       0, 0},
+  {N_("Quit cbrPager"),	"quit_btn",       0, 0},
+  {N_("Rotate"),	"rotate_btn",     0, HILDON_HARDKEY_ESC},
+  {N_("Full screen"),   "fullscreen_btn", 0, HILDON_HARDKEY_FULLSCREEN},
+  {N_("Toggle mode"),   "toggle_keymode", 0, HILDON_HARDKEY_SELECT},
+  {N_("Scroll down"),   "scroll_down",    0, 0},
+  {N_("Scroll up"),     "scroll_up",      0, 0},
+  {N_("Scroll right"),  "scroll_right",   0, 0},
+  {N_("Scroll left"),   "scroll_left",    0, 0},
+  {N_("Page down"),     "page_down",      0, HILDON_HARDKEY_RIGHT},
+  {N_("Page up"),       "page_up",        0, HILDON_HARDKEY_LEFT},
 };
 
 key_assignment key_asg1[] = {
-  {N_("Zoom in"),	"zoom_in_btn",
-			0,		0},
-  {N_("Zoom out"),	"zoom_out_btn",
-			0,		0},
-  {N_("Fit width"),	"zoom_width_btn",
-			0,		0},
-  {N_("Fit page"),	"zoom_fit_btn",
-			0,		0},
-  {N_("Zoom 1:1"),	"zoom_orig_btn",
-			0,		GDK_z},
-  {N_("Next page"),	"next_btn",
-			0,		HILDON_HARDKEY_INCREASE},
-  {N_("Previous page"),	"back_btn",
-			0,		HILDON_HARDKEY_DECREASE},
-  {N_("Last page"),	"last_btn",
-			0,		GDK_l},
-  {N_("First page"),	"begin_btn",
-			0,		GDK_f},
-  {N_("Open file"),	"open_btn",
-			0,	        0},
-  {N_("Quit cbrPager"),	"quit_btn",
-                        0,		0},
-  {N_("Rotate"),	"rotate_btn",
-                        0,		HILDON_HARDKEY_ESC},
-  {N_("Full screen"),    "fullscreen_btn",
-                        0,              HILDON_HARDKEY_FULLSCREEN},
-  {N_("Toggle mode"),   "toggle_btn",
-                        0,              HILDON_HARDKEY_SELECT},
+  {N_("Zoom in"),	"zoom_in_btn",    0, 0},
+  {N_("Zoom out"),	"zoom_out_btn",   0, 0},
+  {N_("Fit width"),	"zoom_width_btn", 0, 0},
+  {N_("Fit page"),	"zoom_fit_btn",   0, 0},
+  {N_("Zoom 1:1"),	"zoom_orig_btn",  0, GDK_z},
+  {N_("Next page"),	"next_btn",       0, HILDON_HARDKEY_INCREASE},
+  {N_("Previous page"),	"back_btn",       0, HILDON_HARDKEY_DECREASE},
+  {N_("Last page"),	"last_btn",       0, GDK_l},
+  {N_("First page"),	"begin_btn",      0, GDK_f},
+  {N_("Open file"),	"open_btn",       0, 0},
+  {N_("Quit cbrPager"),	"quit_btn",       0, 0},
+  {N_("Rotate"),	"rotate_btn",     0, HILDON_HARDKEY_ESC},
+  {N_("Full screen"),   "fullscreen_btn", 0, HILDON_HARDKEY_FULLSCREEN},
+  {N_("Toggle mode"),   "toggle_keymode", 0, HILDON_HARDKEY_SELECT},
+  {N_("Scroll down"),   "scroll_down",    0, HILDON_HARDKEY_DOWN},
+  {N_("Scroll up"),     "scroll_up",      0, HILDON_HARDKEY_UP},
+  {N_("Scroll right"),  "scroll_right",   0, HILDON_HARDKEY_RIGHT},
+  {N_("Scroll left"),   "scroll_left",    0, HILDON_HARDKEY_LEFT},
+  {N_("Page down"),     "page_down",      0, 0},
+  {N_("Page up"),       "page_up",        0, 0},
 };
 
 key_assignment* key_asg = key_asg0;
    
 #else
 key_assignment key_asg0[] = {
-  {N_("Zoom in"),	"zoom_in_btn",
-			0,		GDK_KP_Add},
-  {N_("Zoom out"),	"zoom_out_btn",
-			0,		GDK_KP_Subtract},
-  {N_("Fit width"),	"zoom_width_btn",
-			0,		GDK_w},
-  {N_("Fit page"),	"zoom_fit_btn",
-			0,		GDK_p},
-  {N_("Zoom 1:1"),	"zoom_orig_btn",
-			0,		GDK_1},
-  {N_("Next page"),	"next_btn",
-			0,		GDK_Page_Down},
-  {N_("Previous page"),	"back_btn",
-			0,		GDK_Page_Up},
-  {N_("Last page"),	"last_btn",
-			0,		GDK_End},
-  {N_("First page"),	"begin_btn",
-			0,		GDK_Home},
-  {N_("Open file"),	"open_btn",
-			0,		GDK_o},
-  {N_("Quit cbrPager"),	"quit_btn",
-                        0,		GDK_q},
-  {N_("Rotate"),	"rotate_btn",
-                        0,		GDK_r},
-  {N_("Full screen"),   "fullscreen_btn",
-                        0,              GDK_F11}
+  {N_("Zoom in"),	"zoom_in_btn",	  0, GDK_KP_Add},
+  {N_("Zoom out"),	"zoom_out_btn",	  0, GDK_KP_Subtract},
+  {N_("Fit width"),	"zoom_width_btn", 0, GDK_w},
+  {N_("Fit page"),	"zoom_fit_btn",   0, GDK_p},
+  {N_("Zoom 1:1"),	"zoom_orig_btn",  0, GDK_1},
+  {N_("Next page"),	"next_btn",       0, GDK_Page_Down},
+  {N_("Previous page"),	"back_btn",       0, GDK_Page_Up},
+  {N_("Last page"),	"last_btn",       0, GDK_End},
+  {N_("First page"),	"begin_btn",      0, GDK_Home},
+  {N_("Open file"),	"open_btn",       0, GDK_o},
+  {N_("Quit cbrPager"),	"quit_btn",       0, GDK_q},
+  {N_("Rotate"),	"rotate_btn",     0, GDK_r},
+  {N_("Full screen"),   "fullscreen_btn", 0, GDK_F11},
+  
+  {N_("Scroll down"),   "scroll_down",    0, GDK_Down},
+  {N_("Scroll up"),     "scroll_up",      0, GDK_Up},
+  {N_("Scroll right"),  "scroll_right",   0, GDK_Right},
+  {N_("Scroll left"),   "scroll_left",    0, GDK_Left},
+  {N_("Page down"),     "page_down",      GDK_SHIFT_MASK, GDK_Down},
+  {N_("Page up"),       "page_up",        GDK_SHIFT_MASK, GDK_Up},
 };
 
 key_assignment* key_asg = key_asg0;
@@ -138,83 +115,94 @@ preferences pref = {
   ""
 };
 
+#define KEY_FMT "/%s/accel/%s_key"
+#define MODS_FMT "/%s/accel/%s_mods"
+#define KEY_FMT_ALT "/%s/accel/%s_key_alt"
+#define MODS_FMT_ALT "/%s/accel/%s_mods_alt"
+
+static void
+save_accels(GConfClient* client, 
+	    const char* key_fmt, 
+	    const char* mod_fmt, 
+	    key_assignment* keys)
+{
+  gchar path[120];
+  int ka;
+  
+  for (ka = 0; ka < nr_keys_assigned; ka++) {
+    g_sprintf(path, mod_fmt, prog_name, keys[ka].widget);
+    gconf_client_set_int(client, path, keys[ka].mods, NULL);
+    g_sprintf(path, key_fmt, prog_name, keys[ka].widget);
+    gconf_client_set_int(client, path, keys[ka].accel, NULL);
+  }
+}
+
 void
 save_config(void)
 {
-  char path[120];
+  gchar path[120];
   int ka;
   GConfClient* client;
 
   client = gconf_client_get_default();
 
-  sprintf(path, "/%s/global/lastbook", prog_name);
+  g_sprintf(path, "/%s/global/lastbook", prog_name);
   gconf_client_set_string(client, path, pref.lastbook, NULL);
 
-  sprintf(path, "/%s/global/fixed_dir", prog_name);
+  g_sprintf(path, "/%s/global/fixed_dir", prog_name);
   gconf_client_set_string(client, path, pref.fixeddir, NULL);
 
-  sprintf(path, "/%s/global/nextpage", prog_name);
+  g_sprintf(path, "/%s/global/nextpage", prog_name);
   gconf_client_set_int(client, path, pref.nextpage, NULL);
 
-  sprintf(path, "/%s/global/prevpage", prog_name);
+  g_sprintf(path, "/%s/global/prevpage", prog_name);
   gconf_client_set_int(client, path, pref.prevpage, NULL);
 
-  sprintf(path, "/%s/global/nrpages", prog_name);
+  g_sprintf(path, "/%s/global/nrpages", prog_name);
   gconf_client_set_int(client, path, pref.nrpages, NULL);
 
-  sprintf(path, "/%s/startup/width", prog_name);
+  g_sprintf(path, "/%s/startup/width", prog_name);
   gconf_client_set_int(client, path, pref.initwidth, NULL);
 
-  sprintf(path, "/%s/startup/height", prog_name);
+  g_sprintf(path, "/%s/startup/height", prog_name);
   gconf_client_set_int(client, path, pref.initheight, NULL);
 
-  sprintf(path, "/%s/global/nav_pos", prog_name);
+  g_sprintf(path, "/%s/global/nav_pos", prog_name);
   gconf_client_set_int(client, path, pref.nav_pos, NULL);
 
-  sprintf(path, "/%s/global/nav_hor", prog_name);
+  g_sprintf(path, "/%s/global/nav_hor", prog_name);
   gconf_client_set_int(client, path, pref.nav_hor, NULL);
 
-  sprintf(path, "/%s/global/dirmode", prog_name);
+  g_sprintf(path, "/%s/global/dirmode", prog_name);
   gconf_client_set_int(client, path, pref.dirmode, NULL);
 
-  sprintf(path, "/%s/global/nav_hidden", prog_name);
+  g_sprintf(path, "/%s/global/nav_hidden", prog_name);
   gconf_client_set_int(client, path, pref.nav_hidden, NULL);
 
-  sprintf(path, "/%s/global/pagenumber", prog_name);
+  g_sprintf(path, "/%s/global/pagenumber", prog_name);
   gconf_client_set_int(client, path, pref.pagenumber, NULL);
 
-  sprintf(path, "/%s/global/manga_mode", prog_name);
+  g_sprintf(path, "/%s/global/manga_mode", prog_name);
   gconf_client_set_int(client, path, pref.manga_mode, NULL);
 
-  sprintf(path, "/%s/global/auto_rotate", prog_name);
+  g_sprintf(path, "/%s/global/auto_rotate", prog_name);
   gconf_client_set_int(client, path, pref.auto_rotate, NULL);
 
-  sprintf(path, "/%s/global/zone_size", prog_name);
+  g_sprintf(path, "/%s/global/zone_size", prog_name);
   gconf_client_set_int(client, path, pref.zone_size, NULL);
 
-  sprintf(path, "/%s/global/buffer_size", prog_name);
+  g_sprintf(path, "/%s/global/buffer_size", prog_name);
   gconf_client_set_int(client, path, pref.buffer_size, NULL);
 
-  sprintf(path, "/%s/global/scrollbars_fullscreen", prog_name);
+  g_sprintf(path, "/%s/global/scrollbars_fullscreen", prog_name);
   gconf_client_set_int(client, path, pref.scrollbars_fullscreen, NULL);
 
-  sprintf(path, "/%s/global/rotate_ccw", prog_name);
+  g_sprintf(path, "/%s/global/rotate_ccw", prog_name);
   gconf_client_set_int(client, path, pref.rotate_ccw, NULL);
 
-  for (ka = 0; ka < nr_keys_assigned; ka++) {
-    sprintf(path, "/%s/accel/%s_mods", prog_name, key_asg0[ka].widget);
-    gconf_client_set_int(client, path, key_asg0[ka].mods, NULL);
-    sprintf(path, "/%s/accel/%s_key", prog_name, key_asg0[ka].widget);
-    gconf_client_set_int(client, path, key_asg0[ka].accel, NULL);
-  }
-
+  save_accels(client, KEY_FMT, MODS_FMT, key_asg0);
 #if USE_HILDON
-  for (ka = 0; ka < nr_keys_assigned; ka++) {
-    sprintf(path, "/%s/accel/%s_mods_alt", prog_name, key_asg1[ka].widget);
-    gconf_client_set_int(client, path, key_asg1[ka].mods, NULL);
-    sprintf(path, "/%s/accel/%s_key_alt", prog_name, key_asg1[ka].widget);
-    gconf_client_set_int(client, path, key_asg1[ka].accel, NULL);
-  }
+  save_accels(client, KEY_FMT_ALT, MODS_FMT_ALT, key_asg1);
 #endif
 
   gconf_client_suggest_sync(client, NULL);
@@ -241,6 +229,19 @@ int gconf_client_get_int_with_default(GConfClient* client, const gchar* key, int
   return retval;
 }
 
+static void
+load_accels(GConfClient* client, const char* key_fmt, const char* mod_fmt, key_assignment* keys)
+{
+  gchar path[120];
+  int ka;
+  for (ka = 0; ka < nr_keys_assigned; ka++) {
+    g_sprintf(path, mod_fmt, prog_name, keys[ka].widget);
+    keys[ka].mods = gconf_client_get_int_with_default(client, path, keys[ka].mods);
+    g_sprintf(path, key_fmt, prog_name, keys[ka].widget);
+    keys[ka].accel = gconf_client_get_int_with_default(client, path, keys[ka].accel);
+  }
+}
+
 void
 load_config(void)
 {
@@ -249,6 +250,7 @@ load_config(void)
   GConfClient* client;
 
   client = gconf_client_get_default();
+  gconf_client_suggest_sync(client, NULL);
 
   g_sprintf(path, "/%s/global/lastbook", prog_name);
   strcpy(pref.lastbook, p = gconf_client_get_string_with_default(client, path, ""));
@@ -306,25 +308,12 @@ load_config(void)
   g_sprintf(path, "/%s/global/rotate_ccw", prog_name);
   pref.rotate_ccw = gconf_client_get_int_with_default(client, path, 0);
 
-  if (pref.dirmode == DIRM_FIXED)		// fixed startup directory
-    strncpy(pref.lastbook, pref.fixeddir, sizeof(pref.lastbook));
-  for (ka = 0; ka < nr_keys_assigned; ka++) {
-    g_sprintf(path, "/%s/accel/%s_mods", prog_name, key_asg0[ka].widget);
-    key_asg0[ka].mods = gconf_client_get_int_with_default(client, path, key_asg0[ka].mods);
-    g_sprintf(path, "/%s/accel/%s_key", prog_name, key_asg0[ka].widget);
-    key_asg0[ka].accel = gconf_client_get_int_with_default(client, path, key_asg0[ka].accel);
-  }
 
+  load_accels(client, KEY_FMT, MODS_FMT, key_asg0);
 #if USE_HILDON
-  for (ka = 0; ka < nr_keys_assigned; ka++) {
-    g_sprintf(path, "/%s/accel/%s_mods_alt", prog_name, key_asg1[ka].widget);
-    key_asg1[ka].mods = gconf_client_get_int_with_default(client, path, key_asg1[ka].mods);
-    g_sprintf(path, "/%s/accel/%s_key_alt", prog_name, key_asg1[ka].widget);
-    key_asg1[ka].accel = gconf_client_get_int_with_default(client, path, key_asg1[ka].accel);
-  }
+  load_accels(client, KEY_FMT_ALT, MODS_FMT_ALT, key_asg1);
 #endif
 
-  gconf_client_suggest_sync(client, NULL);
   g_object_unref(G_OBJECT(client));
 }
 
@@ -407,14 +396,33 @@ show_config(GtkWidget *ref)
 }
 
 
-void
-parse_config(GtkWidget *ref)
+static void
+save_accelerators(GtkListStore* store, key_assignment* keys) 
 {
-  GtkWidget *w;
-  GtkTreeIter iter;
   guint accel;
   gint mods;
   int ka;
+  GtkTreeIter iter;
+
+  /* ensure we don't zap configuration with b0rked model values */
+  if(GTK_IS_TREE_MODEL(store)) {
+    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+    for (ka = 0; ka < nr_keys_assigned; ka++) {
+      gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
+			 ACC_MODS, &mods,
+			 ACC_ACCEL, &accel,
+			 -1);
+      keys[ka].mods = mods;
+      keys[ka].accel = accel;
+      gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
+    }
+  }
+}
+
+void
+parse_config(GtkWidget *ref)
+{
+  GtkWidget* w;
 
   w = lookup_widget(ref, "conf_goto_entry");
   strncpy(pref.fixeddir,
@@ -457,9 +465,9 @@ parse_config(GtkWidget *ref)
 
   w = lookup_widget(ref, "conf_prevdir_rbtn");
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
-    pref.dirmode = 0;
+    pref.dirmode = DIRM_LAST;
   else
-    pref.dirmode = 1;
+    pref.dirmode = DIRM_FIXED;
 
   w = lookup_widget(ref, "manga_mode");
   pref.manga_mode = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
@@ -504,24 +512,10 @@ parse_config(GtkWidget *ref)
 		     : GTK_ORIENTATION_VERTICAL);
 #endif
 
-  /* ensure we don't zap configuration with b0rked model values */
-  if(GTK_IS_TREE_MODEL(accel_store)) {
-    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(accel_store), &iter);
-    for (ka = 0; ka < nr_keys_assigned; ka++) {
-      gtk_tree_model_get(GTK_TREE_MODEL(accel_store), &iter,
-			 ACC_MODS, &mods,
-			 ACC_ACCEL, &accel,
-			 -1);
-      w = lookup_widget(nav_window, key_asg0[ka].widget);
-      gtk_widget_remove_accelerator(w, accel_group,
-				    key_asg0[ka].accel, key_asg0[ka].mods);
-      gtk_widget_add_accelerator(w, "clicked", accel_group,
-				 accel, mods, 0);
-      key_asg0[ka].mods = mods;
-      key_asg0[ka].accel = accel;
-      gtk_tree_model_iter_next(GTK_TREE_MODEL(accel_store), &iter);
-    }
-  }
+  save_accelerators(accel_store, key_asg0);
+#if USE_HILDON
+  save_accelerators(special_accel_store, key_asg1);
+#endif
 }
 
 
@@ -530,43 +524,98 @@ conf_accel_tview_cursor_changed(void)
 {
 }
 
+/* methods used for tree view model management */
+static void
+store_accel_model(GtkListStore* store,
+		  GtkTreeIter* iter,
+		  const gchar* legend,
+		  guint accel_key,
+		  gint mods)
+{
+  if(legend)
+    gtk_list_store_set(store, iter,
+		       ACC_FUNCTION, legend,
+		       -1);
+  gtk_list_store_set(store, iter,
+		     ACC_MODS, mods,
+		     ACC_ACCEL, accel_key,
+		     ACC_DISABLE, (mods == 0 && accel_key == 0 ? TRUE : FALSE),
+		     ACC_ESCAPE, (mods == 0 && accel_key == GDK_Escape ? TRUE : FALSE),
+		     -1);
+}
+		  
 
-void
-accel_edited(GtkCellRendererAccel *renderer, gchar *path, guint accel_key,
-		GdkModifierType mask, guint hardware_keycode,
-		GtkTreeView *treeview)
+static void
+update_accel_model(GtkTreeView *treeview, gchar *path,
+		   guint accel_key,
+		   GdkModifierType mask)
 {
   GtkTreeModel *model;
   GtkTreeIter iter;
+  GtkTreePath *tpath;
 
   model = gtk_tree_view_get_model(treeview);
-  if (gtk_tree_model_get_iter_from_string(model, &iter, path))
-    gtk_list_store_set (GTK_LIST_STORE(model), &iter,
-		ACC_MODS, (gint) mask,
-		ACC_ACCEL, accel_key,
-		-1);
+  if (gtk_tree_model_get_iter_from_string(model, &iter, path)) {
+    store_accel_model(GTK_LIST_STORE(model), &iter,
+		      NULL, 
+		      accel_key, 
+		      (gint)mask);
+    tpath = gtk_tree_path_new_from_string(path);
+    gtk_tree_model_row_changed(model, tpath, &iter);
+    gtk_tree_path_free(tpath);
+  }
+}
+
+static void
+accel_edited(GtkCellRendererAccel *renderer, gchar *path, guint accel_key,
+		GdkModifierType mask, guint hardware_keycode,
+		gpointer treeview)
+{
+  update_accel_model(GTK_TREE_VIEW(treeview), path, accel_key, mask);
 }
 
 
-void
-conf_accel_tview_realize(GtkTreeView *tview)
+static void
+disable_accel_toggle(GtkCellRendererToggle *renderer,
+		     gchar *path,
+		     gpointer treeview)
+{
+  update_accel_model(GTK_TREE_VIEW(treeview), path, 0, (GdkModifierType)0);
+}
+
+static void
+escape_accel_toggle(GtkCellRendererToggle *renderer,
+		    gchar *path,
+		    gpointer treeview)
+{
+  update_accel_model(GTK_TREE_VIEW(treeview), path, GDK_Escape, (GdkModifierType)0);
+}
+
+
+static void
+conf_accel_tview_do_realize(GtkTreeView *tview,
+			    GtkListStore **store,
+			    key_assignment* keys) 
 {
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *col;
   GtkTreeIter iter;
   int ka;
 
-  accel_tview = tview;
-  accel_store = gtk_list_store_new(ACC_COLS,
-	G_TYPE_STRING,
-	G_TYPE_INT,
-	G_TYPE_UINT);
+  *store = gtk_list_store_new(ACC_COLS,
+			      G_TYPE_STRING,
+			      G_TYPE_INT,
+			      G_TYPE_UINT,
+			      G_TYPE_BOOLEAN,
+			      G_TYPE_BOOLEAN);
+  
+  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tview), FALSE);
+  gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tview), TRUE);
+  gtk_tree_view_set_model(GTK_TREE_VIEW(tview),
+			  GTK_TREE_MODEL(*store));
+  g_object_unref(G_OBJECT(*store));
 
-  gtk_tree_view_set_model(GTK_TREE_VIEW(accel_tview),                             
-        GTK_TREE_MODEL(accel_store));
-  g_object_unref(G_OBJECT(accel_store));
-
-  // Accelerator function
+  /* Accelerator function */
   renderer = gtk_cell_renderer_text_new();
   gtk_object_set(GTK_OBJECT(renderer),
 		"ypad", 0, NULL);
@@ -575,15 +624,15 @@ conf_accel_tview_realize(GtkTreeView *tview)
 						 "text", 
 						 0, 
 						 NULL);
-  gtk_tree_view_append_column(GTK_TREE_VIEW(accel_tview), col);
+  gtk_tree_view_append_column(GTK_TREE_VIEW(tview), col);
 
-  // Accelerator assignment
+  /* Accelerator assignment */
   renderer = gtk_cell_renderer_accel_new();
-  g_object_set(renderer, "accel-mods", GTK_CELL_RENDERER_ACCEL_MODE_GTK,
+  g_object_set(renderer, "accel-mode", GTK_CELL_RENDERER_ACCEL_MODE_OTHER,
 		"editable", TRUE, NULL);
   g_signal_connect(G_OBJECT(renderer), "accel_edited",
-		G_CALLBACK(accel_edited),
-		(gpointer) accel_tview);
+		   G_CALLBACK(accel_edited),
+		   tview);
   col = gtk_tree_view_column_new_with_attributes(_("Accelerator"), 
 						 renderer,
 						 "accel-mods", 
@@ -591,35 +640,67 @@ conf_accel_tview_realize(GtkTreeView *tview)
 						 "accel-key", 
 						 ACC_ACCEL,
 						 NULL);
-  gtk_tree_view_append_column(GTK_TREE_VIEW(accel_tview), col);
-  gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(accel_tview), TRUE);
+  g_object_set(col, "expand", TRUE, NULL);
+  gtk_tree_view_append_column(GTK_TREE_VIEW(tview), col);
+
+  renderer = gtk_cell_renderer_toggle_new();
+  g_object_set(renderer, "activatable", TRUE, NULL);
+  g_signal_connect(G_OBJECT(renderer), "toggled",
+		   G_CALLBACK(disable_accel_toggle),
+		   tview);
+  col = gtk_tree_view_column_new_with_attributes(_("Disable"),
+						 renderer,
+						 "active",
+						 ACC_DISABLE,
+						 NULL);
+  gtk_tree_view_append_column(GTK_TREE_VIEW(tview), col);
+  
+  renderer = gtk_cell_renderer_toggle_new();
+  g_object_set(renderer, "activatable", TRUE, NULL);
+  g_signal_connect(G_OBJECT(renderer), "toggled",
+		   G_CALLBACK(escape_accel_toggle),
+		   tview);
+  col = gtk_tree_view_column_new_with_attributes(_("Escape"),
+						 renderer,
+						 "active",
+						 ACC_ESCAPE,
+						 NULL);
+  gtk_tree_view_append_column(GTK_TREE_VIEW(tview), col);
+
+  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tview), TRUE);
 
   for (ka = 0; ka < nr_keys_assigned; ka++) {
-    gtk_list_store_append(accel_store, &iter);
-    gtk_list_store_set(accel_store, &iter,
-	ACC_FUNCTION, _(key_asg0[ka].legend),
-	ACC_MODS,       key_asg0[ka].mods,
-	ACC_ACCEL,      key_asg0[ka].accel,
-	-1);
-  }
+    gtk_list_store_append(*store, &iter);
+    store_accel_model(*store, &iter,
+		      keys[ka].legend,
+		      keys[ka].accel,
+		      keys[ka].mods);
+  }  
 }
 
 void
-simulate_accelerator(int keyval, int state)
+conf_accel_tview_realize(GtkTreeView *tview, gboolean main_keys)
+{
+#if USE_HILDON
+  if(main_keys)
+    conf_accel_tview_do_realize(tview, &accel_store, key_asg0);
+  else
+    conf_accel_tview_do_realize(tview, &special_accel_store, key_asg1);
+#else
+  conf_accel_tview_do_realize(tview, &accel_store, key_asg0);
+#endif
+}
+
+const char*
+find_accelerator(int keyval, int state)
 {
   const char* widget_name = NULL;
-  int ka;
-  for(ka = 0; ka < nr_keys_assigned; ++ka) {
-    if(key_asg[ka].accel == keyval && (!key_asg[ka].mods || (state & key_asg[ka].mods) != 0)) {
-      widget_name = key_asg[ka].widget;
-      break;
-    }
-  }
-  if(widget_name) {
-    GtkWidget* widget = lookup_widget(nav_window, widget_name);
-    if(widget)
-      g_signal_emit_by_name(widget, "clicked", NULL);
+  int i;
+  for(i = 0; i < nr_keys_assigned; ++i) {
+    if(key_asg[i].accel == keyval && ((state == 0 && !key_asg[i].mods) || (state & key_asg[i].mods) != 0))
+      return key_asg[i].widget;
   }
+  return NULL;
 }
 
 gboolean
