@@ -22,7 +22,7 @@
 #include <gtk/gtk.h>
 
 #define	prog_name	"cbrPager"
-#define prog_version	"0.9.23"
+#define prog_version	"0.9.24"
 
 #define FOLLOW_TIMER	150
 
@@ -32,9 +32,12 @@ enum {DIRM_LAST, DIRM_FIXED};
 enum {	ACC_FUNCTION,
 	ACC_MODS,
 	ACC_ACCEL,
+	ACC_DISABLE,
+	ACC_ESCAPE,
 	ACC_COLS
 };
 
+
 typedef struct {
   // Preferences that are saved
   char	lastbook[200],
@@ -71,6 +74,6 @@ void		show_config(GtkWidget *ref);
 void		parse_config(GtkWidget *ref);
 
 void		conf_accel_tview_cursor_changed(void);
-void		conf_accel_tview_realize(GtkTreeView *tview);
-void            simulate_accelerator(int keyval, int state);
+void		conf_accel_tview_realize(GtkTreeView *tview, gboolean main_keys);
+const char*     find_accelerator(int keyval, int state);
 gboolean        toggle_accelerator_mode(void);
