@@ -58,7 +58,7 @@ enum FitSettings {
 
 #define ZONE_SIZE 20
 
-GList           *pagelist = NULL;
+GArray          *pagelist = NULL;
 int		 page_nr = 0,
   timer_id = 0;
 #if USE_HILDON
@@ -96,6 +96,8 @@ guint timeout_id = 0;
 
 guchar* image_buffer = NULL;
 
+typedef void (*accel_callback)(void);
+GHashTable* accel_table = NULL;
 
 #ifdef USE_HILDON
 int
@@ -156,6 +158,8 @@ initial_setup(void)
   readers[0] = g_archive_reader_unzip_new(image_buffer, 
 					  pref.buffer_size, 
 					  zip_magic);
+  /* Note: if you have problems with the in-memory unzipper, use this
+     one instead */
   /*  readers[0] = g_archive_reader_command_line_new(image_buffer, 
       pref.buffer_size,
       zip_magic,
@@ -171,6 +175,49 @@ initial_setup(void)
 						 sz_magic,
 						 sz_list_command,
 						 sz_decompress_command);
+
+  accel_table = g_hash_table_new(g_str_hash,
+				 g_str_equal);
+  g_hash_table_insert(accel_table,
+		      "toggle_keymode",
+		      toggle_keymode);
+  g_hash_table_insert(accel_table,
+		      "scroll_down",
+		      scroll_down);
+  g_hash_table_insert(accel_table,
+		      "scroll_up",
+		      scroll_up);
+  g_hash_table_insert(accel_table,
+		      "scroll_right",
+		      scroll_right);
+  g_hash_table_insert(accel_table,
+		      "scroll_left",
+		      scroll_left);
+  g_hash_table_insert(accel_table,
+		      "page_down",
+		      page_down);
+  g_hash_table_insert(accel_table,
+		      "page_up",
+		      page_up);
+}
+
+void
+clean_up(void)
+{
+  int i;
+  if(accel_table) {
+    g_hash_table_unref(accel_table);
+    accel_table = NULL;
+  }
+  reader = NULL;
+  for(i = 0; i < NUM_READERS; ++i) {
+    if(readers[i])
+      g_object_unref(readers[i]);
+    readers[i] = NULL;
+  }
+  if(image_buffer)
+    g_free(image_buffer);
+  image_buffer = NULL;
 }
 
 
@@ -227,360 +274,6 @@ debug_scroll_adj(GtkAdjustment* adj)
 	   adj->value, adj->upper, adj->lower);
 }
 
-void
-scroll_to(double where)
-{
-  GtkWidget *wdg;
-  GtkAdjustment *adj;
-
-  wdg = lookup_widget(MainWindow, "main_scroller");
-  if(rotated) {
-    adj = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(wdg));
-    if(!pref.rotate_ccw)
-      where = 1.0 - where;
-    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj),
-			     where*(adj->upper - adj->page_size));
-    gtk_scrolled_window_set_hadjustment(GTK_SCROLLED_WINDOW(wdg), adj);
-  }
-  else {
-    adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(wdg));
-    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj),
-			     where*(adj->upper - adj->page_size));
-    gtk_scrolled_window_set_vadjustment(GTK_SCROLLED_WINDOW(wdg), adj);
-  }
-  debug_scroll_adj(adj);
-}
-
-
-void
-request_page(int nr, GdkPixbuf **pxm, double *w, double *h)
-{
-  if(reader == NULL) {
-    return;
-  }
-  *pxm = g_archive_reader_get_page(reader, nr, w, h);
-}
-
-
-void
-show_page(int nr)
-{
-  GdkPixbuf *pxm, *pxm1 = NULL, *final = NULL;
-  char *bff;
-  double w, h, w1 = 0, h1 = 0;
-  int nr_pages;
-
-  nr_pages = g_list_length(pagelist);
-
-  if(reader == NULL || !nr_pages) {
-    printf(_("No pages loaded\n"));
-    return;
-  }
-
-#if USE_HILDON
-  bff = g_strdup_printf(_("%s"),
-			(char *)g_list_nth_data(pagelist, nr));
-#else
-  bff = g_strdup_printf(_("cbrPager - %s"),
-			(char *)g_list_nth_data(pagelist, nr));
-#endif
-  gtk_window_set_title(GTK_WINDOW(MainWindow), bff);
-  g_free(bff);
-
-  request_page(nr, &pxm, &w, &h);
-  if(pxm == NULL) {
-    gchar* last_error = g_archive_reader_get_last_error(reader);
-    ok_dialog(_("File error"), last_error);
-    g_free(last_error);
-    return;
-  }
-  if (pref.nrpages == 2 && nr+1 < nr_pages) {
-    int bps, bps1;
-    request_page(nr+1, &pxm1, &w1, &h1);
-    if(pxm1 == NULL) {
-      gchar* last_error = g_archive_reader_get_last_error(reader);
-      ok_dialog(_("File error"), last_error);
-      g_free(last_error);
-      g_object_unref(pxm);
-      return;
-    }
-    bps = gdk_pixbuf_get_bits_per_sample(pxm);
-    bps1 = gdk_pixbuf_get_bits_per_sample(pxm1);
-    
-    final = gdk_pixbuf_new(GDK_COLORSPACE_RGB,
-			   FALSE,
-			   MAX(bps, bps1),
-			   w + w1,
-			   MAX(h, h1));
-    gdk_pixbuf_fill(final, 0xffffffff);
-    if(pref.manga_mode) {
-      gdk_pixbuf_copy_area(pxm1,
-			   0, 0, w1, h1,
-			   final,
-			   0, 0);
-      g_object_unref(pxm1);
-      gdk_pixbuf_copy_area(pxm,
-			   0, 0, w, h,
-			   final,
-			   w1, 0);
-      g_object_unref(pxm);
-    } else {
-      gdk_pixbuf_copy_area(pxm,
-			   0, 0, w, h,
-			   final,
-			   0, 0);
-      g_object_unref(pxm);
-      gdk_pixbuf_copy_area(pxm1,
-			   0, 0, w1, h1,
-			   final,
-			   w, 0);
-      g_object_unref(pxm1);
-    }
-  } else {
-    final = pxm;
-  }
-  
-  
-  total_w = w + w1;
-  total_h = h > h1 ? h : h1;
-  if(pref.auto_rotate && 
-     ((total_w >= total_h && rotated) ||
-      (total_w < total_h && !rotated)))
-    rotate(FALSE);
-
-  gtk_image_view_set_pixbuf(book_canvas, final, zoom_fit == FitSettings_PAGE ? TRUE : FALSE);
-  reset_fit_settings();
-
-  /* now the image view is holding the pixmap, no need to keep it around */
-  g_object_unref(final);
-
-
-  /* save last requested page */
-  pref.pagenumber = nr;
-  save_config();
-}
-
-void
-reset_fit_settings(void)
-{
-  if(zoom_fit == FitSettings_WIDTH)
-    set_zoom_width();
-  else if(zoom_fit == FitSettings_NONE)
-    set_zoom_factor(zoom_factor);
-}
-
-void
-fit_page(int w, int h)
-{
-  gtk_image_view_set_fitting(book_canvas, TRUE);
-  zoom_fit = FitSettings_PAGE;
-}
-
-
-void
-fit_width(int w)
-{
-  double ratio = (double)w / total_w;
-  set_zoom_factor(ratio);
-  /* need to reset zoom_fit because it gets clobbered by set_zoom_factor */
-  zoom_fit = FitSettings_WIDTH;
-}
-
-
-void
-next_page(int adv)
-{
-  int by = adv, 
-    nr_pages = g_list_length(pagelist);
-
-  if (by == 0) by = pref.nrpages;
-  if (page_nr == (nr_pages - by)) {
-    info_dialog(_("End of archive"));
-    return;
-  }
-  page_nr += by;
-
-  show_page(page_nr);
-  switch (pref.nextpage) {
-  case 0: scroll_to(0.0); break;
-  case 1: scroll_to(1.0); break;
-  }
-}
-
-
-void
-previous_page(int prv)
-{
-  int by = prv;
-  
-  if (by == 0) by = pref.nrpages;
-
-  if(page_nr == 0) {
-    info_dialog(_("Beginning of archive"));
-    return;
-  }
-  
-  page_nr -= by;
-  if (page_nr < 0) {
-    page_nr = 0;
-  }
-
-  show_page(page_nr);
-  switch (pref.prevpage) {
-  case 0: scroll_to(0.0); break;
-  case 1: scroll_to(1.0); break;
-  }
-}
-
-
-void
-goto_begin(void)
-{
-  page_nr = 0;
-  show_page(page_nr);
-}
-
-
-void
-goto_end(void)
-{
-  page_nr = g_list_length(pagelist) - pref.nrpages;
-  show_page(page_nr);
-}
-
-
-void
-file_list_expand(gboolean expand)
-{
-  GtkWidget *nav = lookup_widget(nav_window, "nav_toolbar"),
-    *paned;
-  GtkTreeView *tview;
-
-  if (expand) {
-    tview = GTK_TREE_VIEW(gtk_tree_view_new());
-    g_object_ref(G_OBJECT(nav));
-    gtk_container_remove(GTK_CONTAINER(nav_window), nav);
-
-    if (!pref.nav_hor) {
-      paned = gtk_hpaned_new();
-      gtk_widget_show(paned);
-      gtk_container_add(GTK_CONTAINER(nav_window), paned);
-      switch (pref.nav_pos) {
-      case NAV_NW:
-      case NAV_SW:
-	gtk_paned_pack1(GTK_PANED(paned), nav, TRUE, FALSE);
-	gtk_paned_pack2(GTK_PANED(paned), GTK_WIDGET(tview), TRUE, FALSE);
-	break;
-      case NAV_NE:
-      case NAV_SE:
-	gtk_paned_pack2(GTK_PANED(paned), nav, TRUE, FALSE);
-	gtk_paned_pack1(GTK_PANED(paned), GTK_WIDGET(tview), TRUE, FALSE);
-	break;
-      }
-    } else {
-      paned = gtk_vpaned_new();
-      switch (pref.nav_pos) {
-      case NAV_NW:
-      case NAV_NE:
-      case NAV_SW:
-      case NAV_SE:
-	break;
-      }
-    }
-    g_object_unref(G_OBJECT(nav));
-    gtk_widget_show(GTK_WIDGET(tview));
-  } else {
-  }
-  file_list_expanded = expand;
-  printf(_("Expand %d\n"), expand);
-}
-
-
-void
-file_list_tbtn_clicked(void)
-{
-  file_list_expand(!file_list_expanded);
-}
-
-
-void
-update_nav_position(void)
-{
-  int x, y, w, h, nh, nw;
-
-  gtk_window_get_position(GTK_WINDOW(MainWindow), &x, &y);
-
-  gtk_window_set_policy(GTK_WINDOW(nav_window), FALSE, FALSE, TRUE);
-  gtk_toolbar_set_orientation(
-			      GTK_TOOLBAR(lookup_widget(nav_window, "nav_toolbar")),
-			      pref.nav_hor ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL);
-
-  gtk_window_get_size(GTK_WINDOW(MainWindow), &w, &h);
-  gtk_window_get_size(GTK_WINDOW(nav_window), &nw, &nh);
-
-  switch (pref.nav_pos) {
-  case NAV_NW:
-    gtk_window_move(GTK_WINDOW(nav_window), x+5, y+30);
-    break;
-  case NAV_NE:
-    gtk_window_move(GTK_WINDOW(nav_window), x+w-nw-3, y+30);
-    break;
-  case NAV_SE:
-    if (pref.nav_hor)
-      gtk_window_move(GTK_WINDOW(nav_window), x+w-nw-3, y+h-12);
-    else
-      gtk_window_move(GTK_WINDOW(nav_window), x+w-nw-3, y+h-nh);
-    break;
-  case NAV_SW:
-    if (pref.nav_hor)
-      gtk_window_move(GTK_WINDOW(nav_window), x+5, y+h-12); // -nh+10);
-    else
-      gtk_window_move(GTK_WINDOW(nav_window), x+5, y+h-nh);
-    break;
-  }
-}
-
-
-gboolean
-follow_timer(gpointer data)
-{
-  update_nav_position();
-  return TRUE;
-}
-
-
-void
-install_timer(void)
-{
-  timer_id = gtk_timeout_add(FOLLOW_TIMER, follow_timer, NULL);
-}
-
-
-void
-set_zoom_factor(double factor)
-{
-  if(!gtk_image_view_get_fitting(book_canvas) &&
-     zoom_factor == factor)
-    return;
-  gtk_image_view_set_fitting(book_canvas, FALSE);
-  gtk_image_view_set_zoom(book_canvas, factor);
-  zoom_fit = FitSettings_NONE;
-  zoom_factor = factor;
-}
-
-void
-rotate(gboolean reset_fit)
-{
-  if(rotated)
-    gtk_image_view_rotate_cancel(book_canvas);
-  else
-    gtk_image_view_rotate(book_canvas,
-			  pref.rotate_ccw ? 90 : 270);
-  rotated = !rotated;
-  if(reset_fit)
-    reset_fit_settings();
-}
-
 static gboolean
 clear_indicators(gpointer data) {
   if(GTK_IS_WIDGET(data)) {
@@ -604,7 +297,7 @@ draw_indicators(GtkWidget* canvas, int which)
   case 0:
     if(page_nr != 0)
       draw_indicators(canvas, -1);
-    if(page_nr + pref.nrpages < g_list_length(pagelist))
+    if(page_nr + pref.nrpages < pagelist->len)
       draw_indicators(canvas, 1);
     /* set up a timer to remove the arrows after a time */
     if(timeout_id != 0) {
@@ -767,6 +460,363 @@ set_fullscreen_ui(gboolean fullscreen)
 }
 #endif
 
+void
+scroll_to(double where)
+{
+  GtkWidget *wdg;
+  GtkAdjustment *adj;
+
+  wdg = lookup_widget(MainWindow, "main_scroller");
+  if(rotated) {
+    adj = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(wdg));
+    if(!pref.rotate_ccw)
+      where = 1.0 - where;
+    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj),
+			     where*(adj->upper - adj->page_size));
+    gtk_scrolled_window_set_hadjustment(GTK_SCROLLED_WINDOW(wdg), adj);
+  }
+  else {
+    adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(wdg));
+    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj),
+			     where*(adj->upper - adj->page_size));
+    gtk_scrolled_window_set_vadjustment(GTK_SCROLLED_WINDOW(wdg), adj);
+  }
+  debug_scroll_adj(adj);
+}
+
+
+void
+request_page(int nr, GdkPixbuf **pxm, double *w, double *h)
+{
+  if(reader == NULL) {
+    return;
+  }
+  *pxm = g_archive_reader_get_page(reader, nr, w, h);
+}
+
+
+void
+show_page(int nr)
+{
+  GdkPixbuf *pxm, *pxm1 = NULL, *final = NULL;
+  char *bff;
+  double w, h, w1 = 0, h1 = 0;
+  int nr_pages;
+
+  nr_pages = pagelist->len;
+
+  if(reader == NULL || !nr_pages) {
+    printf(_("No pages loaded\n"));
+    return;
+  }
+
+#if USE_HILDON
+  bff = g_strdup_printf(_("%s"),
+			(char *)g_array_index(pagelist, gchar*, nr));
+#else
+  bff = g_strdup_printf(_("cbrPager - %s"),
+			(char *)g_array_index(pagelist, gchar*, nr));
+#endif
+  gtk_window_set_title(GTK_WINDOW(MainWindow), bff);
+  g_free(bff);
+
+  request_page(nr, &pxm, &w, &h);
+  if(pxm == NULL) {
+    gchar* last_error = g_archive_reader_get_last_error(reader);
+    ok_dialog(_("File error"), last_error);
+    g_free(last_error);
+    return;
+  }
+  if (pref.nrpages == 2 && nr+1 < nr_pages) {
+    int bps, bps1;
+    request_page(nr+1, &pxm1, &w1, &h1);
+    if(pxm1 == NULL) {
+      gchar* last_error = g_archive_reader_get_last_error(reader);
+      ok_dialog(_("File error"), last_error);
+      g_free(last_error);
+      g_object_unref(pxm);
+      return;
+    }
+    bps = gdk_pixbuf_get_bits_per_sample(pxm);
+    bps1 = gdk_pixbuf_get_bits_per_sample(pxm1);
+    
+    final = gdk_pixbuf_new(GDK_COLORSPACE_RGB,
+			   FALSE,
+			   MAX(bps, bps1),
+			   w + w1,
+			   MAX(h, h1));
+    gdk_pixbuf_fill(final, 0xffffffff);
+    if(pref.manga_mode) {
+      gdk_pixbuf_copy_area(pxm1,
+			   0, 0, w1, h1,
+			   final,
+			   0, 0);
+      g_object_unref(pxm1);
+      gdk_pixbuf_copy_area(pxm,
+			   0, 0, w, h,
+			   final,
+			   w1, 0);
+      g_object_unref(pxm);
+    } else {
+      gdk_pixbuf_copy_area(pxm,
+			   0, 0, w, h,
+			   final,
+			   0, 0);
+      g_object_unref(pxm);
+      gdk_pixbuf_copy_area(pxm1,
+			   0, 0, w1, h1,
+			   final,
+			   w, 0);
+      g_object_unref(pxm1);
+    }
+  } else {
+    final = pxm;
+  }
+  
+  
+  total_w = w + w1;
+  total_h = h > h1 ? h : h1;
+  if(pref.auto_rotate && 
+     ((total_w >= total_h && rotated) ||
+      (total_w < total_h && !rotated)))
+    rotate(FALSE);
+
+  gtk_image_view_set_pixbuf(book_canvas, final, zoom_fit == FitSettings_PAGE ? TRUE : FALSE);
+  reset_fit_settings();
+
+  /* now the image view is holding the pixmap, no need to keep it around */
+  g_object_unref(final);
+
+
+  /* save last requested page */
+  pref.pagenumber = nr;
+  save_config();
+}
+
+void
+reset_fit_settings(void)
+{
+  if(zoom_fit == FitSettings_WIDTH)
+    set_zoom_width();
+  else if(zoom_fit == FitSettings_NONE)
+    set_zoom_factor(zoom_factor);
+}
+
+void
+fit_page(int w, int h)
+{
+  gtk_image_view_set_fitting(book_canvas, TRUE);
+  zoom_fit = FitSettings_PAGE;
+}
+
+
+void
+fit_width(int w)
+{
+  double ratio = (double)w / total_w;
+  set_zoom_factor(ratio);
+  /* need to reset zoom_fit because it gets clobbered by set_zoom_factor */
+  zoom_fit = FitSettings_WIDTH;
+}
+
+
+void
+next_page(int adv, gboolean show_indicator)
+{
+  int by = adv, nr_pages = pagelist->len;
+
+  if (by == 0) by = pref.nrpages;
+  if (page_nr == (nr_pages - by)) {
+    info_dialog(_("End of archive"));
+    return;
+  }
+  page_nr += by;
+
+  if(show_indicator)
+    draw_indicators(GTK_WIDGET(book_canvas), 1);
+  show_page(page_nr);
+  switch (pref.nextpage) {
+  case 0: scroll_to(0.0); break;
+  case 1: scroll_to(1.0); break;
+  }
+}
+
+
+void
+previous_page(int prv, gboolean show_indicator)
+{
+  int by = prv;
+  
+  if (by == 0) by = pref.nrpages;
+
+  if(page_nr == 0) {
+    info_dialog(_("Beginning of archive"));
+    return;
+  }
+  
+  page_nr -= by;
+  if (page_nr < 0) {
+    page_nr = 0;
+  }
+  
+  if(show_indicator)
+    draw_indicators(GTK_WIDGET(book_canvas), -1);
+  show_page(page_nr);
+  switch (pref.prevpage) {
+  case 0: scroll_to(0.0); break;
+  case 1: scroll_to(1.0); break;
+  }
+}
+
+
+void
+goto_begin(void)
+{
+  page_nr = 0;
+  show_page(page_nr);
+}
+
+
+void
+goto_end(void)
+{
+  page_nr = pagelist->len - pref.nrpages;
+  show_page(page_nr);
+}
+
+
+void
+file_list_expand(gboolean expand)
+{
+  GtkWidget *nav = lookup_widget(nav_window, "nav_toolbar"),
+    *paned;
+  GtkTreeView *tview;
+
+  if (expand) {
+    tview = GTK_TREE_VIEW(gtk_tree_view_new());
+    g_object_ref(G_OBJECT(nav));
+    gtk_container_remove(GTK_CONTAINER(nav_window), nav);
+
+    if (!pref.nav_hor) {
+      paned = gtk_hpaned_new();
+      gtk_widget_show(paned);
+      gtk_container_add(GTK_CONTAINER(nav_window), paned);
+      switch (pref.nav_pos) {
+      case NAV_NW:
+      case NAV_SW:
+	gtk_paned_pack1(GTK_PANED(paned), nav, TRUE, FALSE);
+	gtk_paned_pack2(GTK_PANED(paned), GTK_WIDGET(tview), TRUE, FALSE);
+	break;
+      case NAV_NE:
+      case NAV_SE:
+	gtk_paned_pack2(GTK_PANED(paned), nav, TRUE, FALSE);
+	gtk_paned_pack1(GTK_PANED(paned), GTK_WIDGET(tview), TRUE, FALSE);
+	break;
+      }
+    } else {
+      paned = gtk_vpaned_new();
+      switch (pref.nav_pos) {
+      case NAV_NW:
+      case NAV_NE:
+      case NAV_SW:
+      case NAV_SE:
+	break;
+      }
+    }
+    g_object_unref(G_OBJECT(nav));
+    gtk_widget_show(GTK_WIDGET(tview));
+  } else {
+  }
+  file_list_expanded = expand;
+  printf(_("Expand %d\n"), expand);
+}
+
+
+void
+file_list_tbtn_clicked(void)
+{
+  file_list_expand(!file_list_expanded);
+}
+
+
+void
+update_nav_position(void)
+{
+  int x, y, w, h, nh, nw;
+
+  gtk_window_get_position(GTK_WINDOW(MainWindow), &x, &y);
+
+  gtk_window_set_policy(GTK_WINDOW(nav_window), FALSE, FALSE, TRUE);
+  gtk_toolbar_set_orientation(
+			      GTK_TOOLBAR(lookup_widget(nav_window, "nav_toolbar")),
+			      pref.nav_hor ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL);
+
+  gtk_window_get_size(GTK_WINDOW(MainWindow), &w, &h);
+  gtk_window_get_size(GTK_WINDOW(nav_window), &nw, &nh);
+
+  switch (pref.nav_pos) {
+  case NAV_NW:
+    gtk_window_move(GTK_WINDOW(nav_window), x+5, y+30);
+    break;
+  case NAV_NE:
+    gtk_window_move(GTK_WINDOW(nav_window), x+w-nw-3, y+30);
+    break;
+  case NAV_SE:
+    if (pref.nav_hor)
+      gtk_window_move(GTK_WINDOW(nav_window), x+w-nw-3, y+h-12);
+    else
+      gtk_window_move(GTK_WINDOW(nav_window), x+w-nw-3, y+h-nh);
+    break;
+  case NAV_SW:
+    if (pref.nav_hor)
+      gtk_window_move(GTK_WINDOW(nav_window), x+5, y+h-12); // -nh+10);
+    else
+      gtk_window_move(GTK_WINDOW(nav_window), x+5, y+h-nh);
+    break;
+  }
+}
+
+
+gboolean
+follow_timer(gpointer data)
+{
+  update_nav_position();
+  return TRUE;
+}
+
+
+void
+install_timer(void)
+{
+  timer_id = gtk_timeout_add(FOLLOW_TIMER, follow_timer, NULL);
+}
+
+
+void
+set_zoom_factor(double factor)
+{
+  if(!gtk_image_view_get_fitting(book_canvas) &&
+     zoom_factor == factor)
+    return;
+  gtk_image_view_set_fitting(book_canvas, FALSE);
+  gtk_image_view_set_zoom(book_canvas, factor);
+  zoom_fit = FitSettings_NONE;
+  zoom_factor = factor;
+}
+
+void
+rotate(gboolean reset_fit)
+{
+  if(rotated)
+    gtk_image_view_rotate_cancel(book_canvas);
+  else
+    gtk_image_view_rotate(book_canvas,
+			  pref.rotate_ccw ? 90 : 270);
+  rotated = !rotated;
+  if(reset_fit)
+    reset_fit_settings();
+}
+
 void
 toggle_fullscreen(void)
 {
@@ -860,26 +910,12 @@ set_zoom_width(void)
   fit_width((rotated ? h : w) - adj);
 }
 
-
-void
-clean_up(void)
-{
-  int i;
-  reader = NULL;
-  for(i = 0; i < NUM_READERS; ++i) {
-    if(readers[i])
-      g_object_unref(readers[i]);
-    readers[i] = NULL;
-  }
-  if(image_buffer)
-    g_free(image_buffer);
-  image_buffer = NULL;
-}
-
-
 void
 main_canvas_keyevent(GdkEventKey *event, gboolean is_pressed)
 {
+  const char* accel = NULL;
+  GtkWidget* widget = NULL;
+  accel_callback callback = NULL;
 #if USE_HILDON
   if(event->keyval == HILDON_HARDKEY_MENU) {
     return;
@@ -905,7 +941,25 @@ main_canvas_keyevent(GdkEventKey *event, gboolean is_pressed)
     return;
   }
 #endif
-  simulate_accelerator(event->keyval, event->state);
+  
+  accel = find_accelerator(event->keyval, event->state);
+  if(accel == NULL)
+    return;
+  else if(debug)
+    fprintf(stderr, _("Accel: %s\n"), accel);
+  /* lookup in function table */
+  callback = (accel_callback)g_hash_table_lookup(accel_table,
+						 accel);
+  if(callback)
+    (*callback)();
+  else {
+    /* try to find it in the widgets */
+    widget = lookup_widget(nav_window, accel);
+    if(widget)
+      g_signal_emit_by_name(widget, "clicked", NULL);
+    else
+      fprintf(stderr, _("Unbound event: %s\n"), accel);
+  }
 }
 
 static gboolean
@@ -943,7 +997,7 @@ main_canvas_buttonpressed(GdkEventButton *event)
 
   if (event->type != GDK_BUTTON_PRESS) return;
 
-  nr_pages = g_list_length(pagelist);
+  nr_pages = pagelist->len;
 
   switch (event->button) {
   case 1:				// Left button
@@ -952,16 +1006,10 @@ main_canvas_buttonpressed(GdkEventButton *event)
       x = event->x;
       y = event->y;
       if(is_in_left_zone(x, y, GTK_WIDGET(book_canvas)->allocation)) {
-	if(page_nr > 0) {
-	  draw_indicators(GTK_WIDGET(book_canvas), -1);
-	  previous_page(0);
-	}
+	previous_page(0, TRUE);
 	processed = 1;
       } else if(is_in_right_zone(x, y, GTK_WIDGET(book_canvas)->allocation)) {
-	if(page_nr + pref.nrpages < nr_pages) {
-	  draw_indicators(GTK_WIDGET(book_canvas), 1);
-	  next_page(0);
-	}
+	next_page(0, TRUE);
 	processed = 1;
       }
     }
@@ -1087,7 +1135,7 @@ about_box(GtkWidget* main_window)
 void
 goto_page(GtkWidget* main_window)
 {
-  int nr_pages = g_list_length(pagelist);
+  int nr_pages = pagelist->len;
   int result = invoke_goto_page_dialog(main_window, 
 				       page_nr, 
 				       nr_pages - pref.nrpages);
@@ -1113,3 +1161,79 @@ toggle_rotation_direction(void)
     gtk_image_view_rotate(book_canvas, 180);
   }
 }
+
+void
+scroll_down(void)
+{
+  g_signal_emit_by_name(book_canvas,
+			"scroll",
+			GTK_SCROLL_NONE,
+			GTK_SCROLL_STEP_DOWN);
+}
+
+void
+scroll_up(void)
+{
+  g_signal_emit_by_name(book_canvas,
+			"scroll",
+			GTK_SCROLL_NONE,
+			GTK_SCROLL_STEP_UP);
+}
+
+void
+scroll_right(void)
+{
+  g_signal_emit_by_name(book_canvas,
+			"scroll",
+			GTK_SCROLL_STEP_RIGHT,
+			GTK_SCROLL_NONE);
+}
+
+void
+scroll_left(void)
+{
+  g_signal_emit_by_name(book_canvas,
+			"scroll",
+			GTK_SCROLL_STEP_LEFT,
+			GTK_SCROLL_NONE);
+}
+
+void
+page_down(void)
+{
+  GtkScrollType h, v;
+  if(rotated) {
+    if(pref.rotate_ccw)
+      h = GTK_SCROLL_PAGE_RIGHT;
+    else
+      h = GTK_SCROLL_PAGE_LEFT;
+    v = GTK_SCROLL_NONE;
+  } else {
+    h = GTK_SCROLL_NONE;
+    v = GTK_SCROLL_PAGE_DOWN;
+  }
+  g_signal_emit_by_name(book_canvas,
+			"scroll",
+			h,
+			v);
+}
+
+void 
+page_up(void)
+{
+  GtkScrollType h, v;
+  if(rotated) {
+    if(pref.rotate_ccw)
+      h = GTK_SCROLL_PAGE_LEFT;
+    else
+      h = GTK_SCROLL_PAGE_RIGHT;
+    v = GTK_SCROLL_NONE;
+  } else {
+    h = GTK_SCROLL_NONE;
+    v = GTK_SCROLL_PAGE_UP;
+  }
+  g_signal_emit_by_name(book_canvas,
+			"scroll",
+			h,
+			v);
+}
