@@ -1174,75 +1174,6 @@ gtk_image_view_class_init (GtkImageViewClass *klass)
                       G_TYPE_NONE,
                       2, GTK_TYPE_ADJUSTMENT, GTK_TYPE_ADJUSTMENT);
     klass->set_scroll_adjustments  = gtk_image_view_set_scroll_adjustments;
-
-    /* Add keybindings. */
-    GtkBindingSet *binding_set = gtk_binding_set_by_class (klass);
-
-    /* Unmodified scrolling */
-    gtk_binding_entry_add_signal (binding_set, GDK_Right, 0,
-                                  "scroll", 2,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_STEP_RIGHT,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_NONE);
-    gtk_binding_entry_add_signal (binding_set, GDK_Left, 0,
-                                  "scroll", 2,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_STEP_LEFT,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_NONE);
-    gtk_binding_entry_add_signal (binding_set, GDK_Down, 0,
-                                  "scroll", 2,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_NONE,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_STEP_DOWN);
-    gtk_binding_entry_add_signal (binding_set, GDK_Up, 0,
-                                  "scroll", 2,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_NONE,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_STEP_UP);
-
-    /* Shifted scrolling */
-    gtk_binding_entry_add_signal (binding_set, GDK_Right, GDK_SHIFT_MASK,
-                                  "scroll", 2,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_PAGE_RIGHT,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_NONE);
-    gtk_binding_entry_add_signal (binding_set, GDK_Left, GDK_SHIFT_MASK, 
-                                  "scroll", 2,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_PAGE_LEFT,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_NONE);
-    gtk_binding_entry_add_signal (binding_set, GDK_Up, GDK_SHIFT_MASK,
-                                  "scroll", 2,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_NONE,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_PAGE_UP);
-    gtk_binding_entry_add_signal (binding_set, GDK_Down, GDK_SHIFT_MASK,
-                                  "scroll", 2,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_NONE,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_PAGE_DOWN);
-
-    /* Page Up & Down */
-    gtk_binding_entry_add_signal (binding_set, GDK_Page_Up, 0,
-                                  "scroll", 2,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_NONE,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_PAGE_UP);
-    gtk_binding_entry_add_signal (binding_set, GDK_Page_Down, 0,
-                                  "scroll", 2,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_NONE,
-                                  GTK_TYPE_SCROLL_TYPE,
-                                  GTK_SCROLL_PAGE_DOWN);
 }
 
 /**
